# 自动装箱和拆箱

> 原文： [https://docs.oracle.com/javase/tutorial/java/data/autoboxing.html](https://docs.oracle.com/javase/tutorial/java/data/autoboxing.html)

_Autoboxing_ 是 Java 编译器在原始类型和相应的对象包装类之间进行的自动转换。例如，将 `int` 转换为`整数`，`双`转换为 `Double` ，依此类推。如果转换是另一种方式，则称为 _ 拆箱 _。

以下是自动装箱的最简单示例：

```
Character ch = 'a';

```

本节中的其余示例使用泛型。如果您还不熟悉泛型的语法，请参阅 [Generics（更新）](../generics/index.html)课程。

请考虑以下代码：

```
List<Integer> li = new ArrayList<>();
for (int i = 1; i < 50; i += 2)
    li.add(i);

```

虽然您将 `int` 值作为基本类型而不是 `Integer` 对象添加到 `li` ，但代码会编译。因为 `li` 是 `Integer` 对象的列表，而不是 `int` 值的列表，您可能想知道为什么 Java 编译器不会发出编译时错误。编译器不会生成错误，因为它会从 `i` 创建 `Integer` 对象，并将对象添加到 `li` 。因此，编译器在运行时将以前的代码转换为以下代码：

```
List<Integer> li = new ArrayList<>();
for (int i = 1; i < 50; i += 2)
    li.add(Integer.valueOf(i));

```

将原始值（例如 `int` ）转换为相应包装类（ `Integer` ）的对象称为自动装箱。当原始值为：时，Java 编译器应用自动装箱：

*   作为参数传递给期望相应包装类的对象的方法。
*   分配给相应包装类的变量。

请考虑以下方法：

```
public static int sumEven(List<Integer> li) {
    int sum = 0;
    for (Integer i: li)
        if (i % 2 == 0)
            sum += i;
        return sum;
}

```

因为余数（`％`）和一元加（ `+ =` ）运算符不适用于 `Integer` 对象，您可能想知道为什么 Java 编译器在没有编译方法的情况下编译发出任何错误。编译器不会生成错误，因为它调用 `intValue` 方法在运行时将 `Integer` 转换为 `int` ：

```
public static int sumEven(List<Integer> li) {
    int sum = 0;
    for (Integer i : li)
        if (i.intValue() % 2 == 0)
            sum += i.intValue();
        return sum;
}

```

将包装类型（ `Integer` ）的对象转换为其对应的原始（ `int` ）值称为拆箱。当包装类的对象是：时，Java 编译器应用拆箱：

*   作为参数传递给期望相应基元类型的值的方法。
*   分配给相应基元类型的变量。

[``拆箱`` ](examples/Unboxing.java)示例显示了这是如何工作的：

```
import java.util.ArrayList;
import java.util.List;

public class Unboxing {

    public static void main(String[] args) {
        Integer i = new Integer(-8);

        // 1\. Unboxing through method invocation
        int absVal = absoluteValue(i);
        System.out.println("absolute value of " + i + " = " + absVal);

        List<Double> ld = new ArrayList<>();
        ld.add(3.1416);    // Π is autoboxed through method invocation.

        // 2\. Unboxing through assignment
        double pi = ld.get(0);
        System.out.println("pi = " + pi);
    }

    public static int absoluteValue(int i) {
        return (i < 0) ? -i : i;
    }
}

```

该程序打印以下内容：

```
absolute value of -8 = 8
pi = 3.1416

```

自动装箱和拆箱使开发人员可以编写更清晰的代码，使其更易于阅读。下表列出了原始类型及其相应的包装类，Java 编译器使用这些类进行自动装箱和取消装箱：

| 原始类型 | 包装类 |
| --- | --- |
| 布尔 | 布尔 |
| 字节 | 字节 |
| 烧焦 | 字符 |
| 浮动 | 浮动 |
| INT | 整数 |
| 长 | 长 |
| 短 | 短 |
| 双 | 双 |