# 原始类型

> 原文： [https://docs.oracle.com/javase/tutorial/java/generics/rawTypes.html](https://docs.oracle.com/javase/tutorial/java/generics/rawTypes.html)

_ 原始类型 _ 是没有任何类型参数的泛型类或接口的名称。例如，给定通用 `Box` 类：

```
public class Box<T> {
    public void set(T t) { /* ... */ }
    // ...
}

```

创建参数化类型的 `Box&lt; T&gt;` ，为正式类型参数 `T` 提供实际类型参数：

```
Box<Integer> intBox = new Box<>();

```

如果省略实际类型参数，则创建 `Box&lt; T&gt;的原始类型。` ：

```
Box rawBox = new Box();

```

因此， `Box` 是泛型类型 `Box&lt; T&gt;的原始类型。` 。但是，非泛型类或接口类型是 _ 而不是 _ 原始类型。

原始类型显示在遗留代码中，因为许多 API 类（例如`集合`类）在 JDK 5.0 之前不是通用的。当使用原始类型时，你基本上得到了前仿制行为 - `Box` 给你`对象`。为了向后兼容，允许将参数化类型分配给其原始类型：

```
Box<String> stringBox = new Box<>();
Box rawBox = stringBox;               // OK

```

但是，如果将原始类型分配给参数化类型，则会收到警告：

```
Box rawBox = new Box();           // rawBox is a raw type of Box<T>
Box<Integer> intBox = rawBox;     // warning: unchecked conversion

```

如果使用原始类型调用相应泛型类型中定义的泛型方法，也会收到警告：

```
Box<String> stringBox = new Box<>();
Box rawBox = stringBox;
rawBox.set(8);  // warning: unchecked invocation to set(T)

```

警告显示原始类型绕过泛型类型检查，将不安全代码的捕获推迟到运行时。因此，您应该避免使用原始类型。

[类型擦除](erasure.html)部分提供了有关 Java 编译器如何使用原始类型的更多信息。

## 未选中的错误消息

如前所述，在将遗留代码与通用代码混合时，您可能会遇到类似于以下内容的警告消息：

```
Note: Example.java uses unchecked or unsafe operations.
Note: Recompile with -Xlint:unchecked for details.

```

使用对原始类型进行操作的旧 API 时会发生这种情况，如以下示例所示：

```
public class WarningDemo {
    public static void main(String[] args){
        Box<Integer> bi;
        bi = createBox();
    }

    static Box createBox(){
        return new Box();
    }
}

```

术语“未选中”表示编译器没有足够的类型信息来执行确保类型安全所必需的所有类型检查。默认情况下，“unchecked”警告被禁用，尽管编译器提供了提示。要查看所有“未选中”警告，请使用 `-Xlint 重新编译：取消选中`。

用 `-Xlint 重新编译前面的例子：unchecked` 显示以下附加信息：

```
WarningDemo.java:4: warning: [unchecked] unchecked conversion
found   : Box
required: Box<java.lang.Integer>
        bi = createBox();
                      ^
1 warning

```

要完全禁用未检查的警告，请使用 `-Xlint：-unchecked` 标志。 `@SuppressWarnings（“未选中”）`注释会抑制未经检查的警告。如果您不熟悉`@SuppressWarnings`语法，请参阅[注释](../../java/annotations/index.html)。