# 有界类型参数

> 原文： [https://docs.oracle.com/javase/tutorial/java/generics/bounded.html](https://docs.oracle.com/javase/tutorial/java/generics/bounded.html)

有时您可能希望限制可在参数化类型中用作类型参数的类型。例如，对数字进行操作的方法可能只想接受`Number`或其子类的实例。这是 _ 有界类型参数 _ 的用途。

要声明有界类型参数，请列出类型参数的名称，后跟`extends`关键字，然后是 _ 上限 _，在此示例中为`Number`。请注意，在此上下文中，`extends`在一般意义上用于表示“扩展”（如在类中）或“实现”（如在接口中）。

```
public class Box<T> {

    private T t;          

    public void set(T t) {
        this.t = t;
    }

    public T get() {
        return t;
    }

    public <U extends Number> void inspect(U u){
        System.out.println("T: " + t.getClass().getName());
        System.out.println("U: " + u.getClass().getName());
    }

    public static void main(String[] args) {
        Box<Integer> integerBox = new Box<Integer>();
        integerBox.set(new Integer(10));
        integerBox.inspect("some text"); // error: this is still String!
    }
}

```

通过修改我们的泛型方法来包含这个有界类型参数，编译现在将失败，因为我们的`inspect`调用仍然包含`String`：

```
Box.java:21: <U>inspect(U) in Box<java.lang.Integer> cannot
  be applied to (java.lang.String)
                        integerBox.inspect("10");
                                  ^
1 error

```

除了限制可用于实例化泛型类型的类型之外，有界类型参数还允许您调用边界中定义的方法：

```
public class NaturalNumber<T extends Integer> {

    private T n;

    public NaturalNumber(T n)  { this.n = n; }

    public boolean isEven() {
        return n.intValue() % 2 == 0;
    }

    // ...
}

```

`isEven` 方法通过 `n` 调用`整数`类中定义的 `intValue` 方法。

## 多个边界

前面的示例说明了使用带有单个边界的类型参数，但是类型参数可以具有 _ 多个边界 _：

```
<T extends B1 & B2 & B3>

```

具有多个边界的类型变量是绑定中列出的所有类型的子类型。如果其中一个边界是类，则必须首先指定它。例如：

```
Class A { /* ... */ }
interface B { /* ... */ }
interface C { /* ... */ }

class D <T extends A & B & C> { /* ... */ }

```

如果未首先指定绑定 `A` ，则会出现编译时错误：

```
class D <T extends B & A & C> { /* ... */ }  // compile-time error

```