# 什么是类？

> 原文： [https://docs.oracle.com/javase/tutorial/java/concepts/class.html](https://docs.oracle.com/javase/tutorial/java/concepts/class.html)

在现实世界中，您经常会发现许多相同类型的单个对象。可能存在数千种其他自行车，所有相同的品牌和型号。每辆自行车都是使用相同的蓝图构建的，因此包含相同的组件。在面向对象的术语中，我们说你的自行车是类物体的 _ 实例 _，被称为自行车。 _ 类 _ 是创建单个对象的蓝图。

以下 [`Bicycle`](examples/Bicycle.java) 类是自行车的一种可能实现方式：

```
class Bicycle {

    int cadence = 0;
    int speed = 0;
    int gear = 1;

    void changeCadence(int newValue) {
         cadence = newValue;
    }

    void changeGear(int newValue) {
         gear = newValue;
    }

    void speedUp(int increment) {
         speed = speed + increment;   
    }

    void applyBrakes(int decrement) {
         speed = speed - decrement;
    }

    void printStates() {
         System.out.println("cadence:" +
             cadence + " speed:" + 
             speed + " gear:" + gear);
    }
}

```

Java 编程语言的语法对您来说很新，但这个类的设计基于之前对自行车对象的讨论。字段`cadence`，`speed`和`gear`表示对象的状态，方法（`changeCadence`，`changeGear`，`speedUp`等）定义其与外部世界的交互。

您可能已经注意到`Bicycle`类不包含`main`方法。那是因为它不是一个完整的应用程序;它只是自行车的蓝图，可能是 _ 在应用程序中使用 _。创建和使用新`Bicycle`对象的责任属于应用程序中的其他类。

这是一个 [`BicycleDemo`](examples/BicycleDemo.java) 类，它创建两个独立的`Bicycle`对象并调用它们的方法：

```
class BicycleDemo {
    public static void main(String[] args) {

        // Create two different 
        // Bicycle objects
        Bicycle bike1 = new Bicycle();
        Bicycle bike2 = new Bicycle();

        // Invoke methods on 
        // those objects
        bike1.changeCadence(50);
        bike1.speedUp(10);
        bike1.changeGear(2);
        bike1.printStates();

        bike2.changeCadence(50);
        bike2.speedUp(10);
        bike2.changeGear(2);
        bike2.changeCadence(40);
        bike2.speedUp(10);
        bike2.changeGear(3);
        bike2.printStates();
    }
}

```

此测试的输出打印两辆自行车的结束踏板节奏，速度和档位：

```
cadence:50 speed:10 gear:2
cadence:40 speed:20 gear:3

```