# 部署无代码库

> 原文： [https://docs.oracle.com/javase/tutorial/deployment/deploymentInDepth/deployingWithoutCodebase.html](https://docs.oracle.com/javase/tutorial/deployment/deploymentInDepth/deployingWithoutCodebase.html)

从 Java SE 7 发行版开始，您不必为 Java Web Start 应用程序的 Java 网络启动协议（JNLP）文件中的`codebase`属性指定绝对路径。您可以在不同环境中开发和测试应用程序，而无需修改`codebase`属性中的路径。如果未指定代码库，则 Java Web Start 软件假定代码库与启动 Java Web Start 应用程序的 Web 页面相关。

当 JNLP 文件不包含`codebase`属性时，可以使用 Deployment Toolkit 脚本的以下功能在 Web 页面中部署 Java Web Start 应用程序：

*   [`launchWebStartApplication`](#launchWebStartApplication) - 在 HTML 链接中使用此功能来部署 Java Web Start 应用程序。
*   [`createWebStartLaunchButtonEx`](#createWebStartLaunchButtonEx) - 使用此功能为 Java Web Start 应用程序创建启动按钮。

* * *

**Note:** To run, Java Web Start applications deployed by using the previously specified functions require at least the Java SE 7 release. If the client does not have at least the Java SE 7 release, the functions instruct the user to install the required Java Runtime Environment (JRE) software before launching the Java Web Start application.

* * *

**功能签名：** `launchWebStartApplication: function(jnlp)`

参数：

`jnlp` - 包含 Java Web Start 应用程序部署信息的 JNLP 文件的路径。此路径可以与部署 Java Web Start 应用程序的网页相关。

用法：

在以下示例中，在`anchor (a)`标记的`href`属性中调用`launchWebStartApplication`函数。

[``dynamictree_webstart_no_codebase.jnlp`` ](../webstart/examples/webstart_ComponentArch_DynamicTreeDemo/src/dynamictree_webstart_no_codebase.jnlp)JNLP 文件用于部署动态树演示应用程序。

```
<script src="https://www.java.com/js/deployJava.js"></script>
<a href="javascript:deployJava.launchWebStartApplication('dynamictree_webstart_no_codebase.jnlp');">Launch</a>

```

当用户单击生成的 HTML 链接时，将启动 Java Web Start 应用程序。

**功能签名：** `createWebStartLaunchButtonEx: function(jnlp)`

参数：

`jnlp` - 包含 Java Web Start 应用程序部署信息的 JNLP 文件的路径。此路径可以与部署 Java Web Start 应用程序的网页相关。

用法：

以下示例显示`createWebStartLaunchButtonEx`功能的用法。

[``dynamictree_webstart_no_codebase.jnlp`` ](../webstart/examples/webstart_ComponentArch_DynamicTreeDemo/src/dynamictree_webstart_no_codebase.jnlp)JNLP 文件用于部署动态树演示应用程序。

```
<script src="https://www.java.com/js/deployJava.js"></script>
<script>        
    var jnlpFile = "dynamictree_webstart_no_codebase.jnlp";
    deployJava.createWebStartLaunchButtonEx(jnlpFile);
</script>

```

当用户单击生成的“启动”按钮时，将启动 Java Web Start 应用程序。

在浏览器中打开 [``JavaWebStartAppPage_No_Codebase.html``](../webstart/examples/dist/webstart_ComponentArch_DynamicTreeDemo/JavaWebStartAppPage_No_Codebase.html)，以查看使用本主题中描述的功能部署的动态树演示应用程序。

* * *

**Note:** 

您还可以通过使用 JNLP 文件的完整 URL 调用`javaws`命令在系统命令提示符下启动 Java Web Start 应用程序，如以下代码段所示。

```
javaws http://example.com/dynamictree_webstart_no_codebase.jnlp

```

* * *

[下载 _ 动态树演示 _ 示例的源代码](../webstart/examplesIndex.html#DynamicTreeDemo)以进一步试验。