# 自定义归类规则

> 原文： [https://docs.oracle.com/javase/tutorial/i18n/text/rule.html](https://docs.oracle.com/javase/tutorial/i18n/text/rule.html)

上一节讨论了如何使用区域设置的预定义规则来比较字符串。这些归类规则确定字符串的排序顺序。如果预定义的排序规则不符合您的需要，您可以设计自己的规则并将它们分配给`RuleBasedCollator`对象。

自定义归类规则包含在传递给`RuleBasedCollator`构造函数的`String`对象中。这是一个简单的例子：

```
String simpleRule = "< a < b < c < d";
RuleBasedCollator simpleCollator =  new RuleBasedCollator(simpleRule);

```

对于前一个示例中的`simpleCollator`对象，`a`小于`b`，小于`c`，依此类推。比较字符串时，`simpleCollator.compare`方法引用这些规则。用于构造排序规则的完整语法比这个简单示例更灵活，更复杂。有关语法的完整说明，请参阅 [`RuleBasedCollator`](https://docs.oracle.com/javase/8/docs/api/java/text/RuleBasedCollator.html) 类的 API 文档。

下面的示例使用两个 collat​​ors 对西班牙语单词列表进行排序。此示例的完整源代码位于 [`RulesDemo.java`](examples/RulesDemo.java) 中。

`RulesDemo`程序首先定义英语和西班牙语的校对规则。该计划将以传统方式对西班牙语单词进行排序。当按照传统规则进行排序时，字母 ch 和 ll 及其大写等价物在排序顺序中各自具有自己的位置。这些字符对比较它们就好像是一个字符。例如，ch 按照排序顺序中的 cz 排序为单个字母。请注意两个合作者的规则有何不同：

```
String englishRules = (
    "< a,A < b,B < c,C < d,D < e,E < f,F " +
    "< g,G < h,H < i,I < j,J < k,K < l,L " +
    "< m,M < n,N < o,O < p,P < q,Q < r,R " +
    "< s,S < t,T < u,U < v,V < w,W < x,X " +
    "< y,Y < z,Z");

String smallnTilde = new String("\u00F1");    // ñ
String capitalNTilde = new String("\u00D1");  // Ñ

String traditionalSpanishRules = (
    "< a,A < b,B < c,C " +
    "< ch, cH, Ch, CH " +
    "< d,D < e,E < f,F " +
    "< g,G < h,H < i,I < j,J < k,K < l,L " +
    "< ll, lL, Ll, LL " +
    "< m,M < n,N " +
    "< " + smallnTilde + "," + capitalNTilde + " " +
    "< o,O < p,P < q,Q < r,R " +
    "< s,S < t,T < u,U < v,V < w,W < x,X " +
    "< y,Y < z,Z");

```

以下代码行创建了 collat​​ors 并调用了 sort 例程：

```
try {
    RuleBasedCollator enCollator = new RuleBasedCollator(englishRules);
    RuleBasedCollator spCollator =
        new RuleBasedCollator(traditionalSpanishRules);

    sortStrings(enCollator, words);
    printStrings(words);
    System.out.println();

    sortStrings(spCollator, words);
    printStrings(words);
} catch (ParseException pe) {
    System.out.println("Parse exception for rules");
}

```

称为`sortStrings`的排序例程是通用的。它将根据任何`Collator`对象的规则对任何单词数组进行排序：

```
public static void sortStrings(Collator collator, String[] words) {
    String tmp;
    for (int i = 0; i < words.length; i++) {
        for (int j = i + 1; j < words.length; j++) {
            if (collator.compare(words[i], words[j]) > 0) {
                tmp = words[i];
                words[i] = words[j];
                words[j] = tmp;
            }
        }
    }
}

```

使用英语排序规则排序时，单词数组如下：

```
chalina
curioso
llama
luz

```

将前面的列表与以下列表进行比较，该列表根据传统的西班牙语排序规则进行排序：

```
curioso
chalina
luz
llama

```