# 解析 URL

> 原文： [https://docs.oracle.com/javase/tutorial/networking/urls/urlInfo.html](https://docs.oracle.com/javase/tutorial/networking/urls/urlInfo.html)

`URL`类提供了几种方法，可以让您查询`URL`对象。您可以使用以下访问器方法从 URL 获取协议，权限，主机名，端口号，路径，查询，文件名和引用：

**`getProtocol`**

Returns the protocol identifier component of the URL.

**`getAuthority`**

Returns the authority component of the URL.

**`getHost`**

Returns the host name component of the URL.

**`getPort`**

Returns the port number component of the URL. The `getPort` method returns an integer that is the port number. If the port is not set, `getPort` returns -1.

**`getPath`**

Returns the path component of this URL.

**`getQuery`**

Returns the query component of this URL.

**`getFile`**

Returns the filename component of the URL. The `getFile` method returns the same as `getPath`, plus the concatenation of the value of `getQuery`, if any.

**`getRef`**

Returns the reference component of the URL.

* * *

**Note:** 

请记住，并非所有 URL 地址都包含这些组件。 URL 类提供了这些方法，因为 HTTP URL 确实包含这些组件，并且可能是最常用的 URL。 URL 类有点以 HTTP 为中心。

* * *

您可以使用这些 `get _XXX_`方法获取有关 URL 的信息，而不管您用于创建 URL 对象的构造函数。

URL 类以及这些访问器方法使您不必再次解析 URL！给定 URL 的任何字符串规范，只需创建一个新的 URL 对象并调用任何访问器方法以获取所需的信息。这个小示例程序从字符串规范创建一个 URL，然后使用 URL 对象的访问器方法来解析 URL：

```
import java.net.*;
import java.io.*;

public class ParseURL {
    public static void main(String[] args) throws Exception {

        URL aURL = new URL("http://example.com:80/docs/books/tutorial"
                           + "/index.html?name=networking#DOWNLOADING");

        System.out.println("protocol = " + aURL.getProtocol());
        System.out.println("authority = " + aURL.getAuthority());
        System.out.println("host = " + aURL.getHost());
        System.out.println("port = " + aURL.getPort());
        System.out.println("path = " + aURL.getPath());
        System.out.println("query = " + aURL.getQuery());
        System.out.println("filename = " + aURL.getFile());
        System.out.println("ref = " + aURL.getRef());
    }
}

```

这是程序显示的输出：

```
protocol = http
authority = example.com:80
host = example.com
port = 80
path = /docs/books/tutorial/index.html
query = name=networking
filename = /docs/books/tutorial/index.html?name=networking
ref = DOWNLOADING

```